package org.dataone.tidy;

import java.io.File;
import java.io.FilenameFilter;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;
import org.apache.commons.configuration.Configuration;
import org.dataone.client.auth.CertificateManager;
import org.dataone.cn.dao.exceptions.DataAccessException;
import org.dataone.configuration.Settings;
import org.dataone.service.exceptions.NotImplemented;
import org.dataone.service.exceptions.ServiceFailure;
import org.dataone.tidy.certificate.CertificateFilter;
import org.dataone.tidy.concurrent.TidyJobFactory;
import org.dataone.tidy.dao.ChangeRecordRepository;
import org.dataone.tidy.dao.MergeResultRepository;
import org.dataone.tidy.dao.PostgresRepositoryConfiguration;
import org.dataone.tidy.dao.SystemMetadataTidyDao;
import org.dataone.tidy.dao.SystemMetadataTidyDaoImpl;
import org.dataone.tidy.merge.AuthMNSystemMetadataProxy;
import org.dataone.tidy.merge.MNSystemMetadataDelegate;
import org.dataone.tidy.merge.MergeExecutorService;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import ch.qos.logback.classic.LoggerContext;
import ch.qos.logback.core.util.StatusPrinter;

/**
 * tidy up some node systemMetadata NOW!!!!
 *
 */

// D1 Client will use the default D1Client.CN_URL property in d1client.properties

public class Main {
    private static final Logger logger = LoggerFactory.getLogger(Main.class);

    private static FilenameFilter certificateFilter = new CertificateFilter();
    private static File[] certificateFiles;
    private static Configuration settingsConfiguration = null;

    private static PostgresRepositoryConfiguration repositoryConfig = new PostgresRepositoryConfiguration();
    private static MergeResultRepository mergeResultsRepository;
    private static ChangeRecordRepository changeRecordRepository;
    private static SystemMetadataTidyDao systemMetadataDao = new SystemMetadataTidyDaoImpl();
    private static MNSystemMetadataDelegate mnReadClient = new AuthMNSystemMetadataProxy();

    static {
        settingsConfiguration = Settings.getConfiguration();
        String clientCertLocation = settingsConfiguration
                .getString("D1Client.certificate.directory");
        String clientCertFilename = settingsConfiguration
                .getString("D1Client.certificate.filename");
        File d1CertFile = new File(clientCertLocation + File.separator + clientCertFilename);
        String clientCertificateLocation = d1CertFile.getAbsolutePath();
        CertificateManager.getInstance().setCertificateLocation(clientCertificateLocation);
        
        repositoryConfig.initContext();
        mergeResultsRepository = repositoryConfig.getMergeResultRepository();
        changeRecordRepository = repositoryConfig.getChangeRecordRepository();
    }

    public static void main(String[] args) throws NotImplemented, ServiceFailure,
            DataAccessException {
        try {
            // print internal state
            LoggerContext lc = (LoggerContext) LoggerFactory.getILoggerFactory();
            StatusPrinter.print(lc);


            String method = null;
            // create the command line parser
            CommandLineParser parser = new PosixParser();
            HelpFormatter formatter = new HelpFormatter();
            // create the Options
            Options options = new Options();

            options.addOption("h", "help", false, "print options");

            options.addOption(OptionBuilder
                    .withLongOpt("method")
                    .withDescription(
                            "the name of the method[harvest|merge|commit|evict] to execute")
                    .hasArg().withType(String.class).withValueSeparator().withArgName("METHOD")
                    .create());

            CommandLine line = parser.parse(options, args);
            if (line.hasOption("h") || line.hasOption("help")) {

                formatter.printHelp("tidy-node-sysmeta", options);
                return;
            }

            if (line.hasOption("method")) {
                method = line.getOptionValue("method");
            } else {
                formatter.printHelp("tidy-node-sysmeta", options);
                return;
            }
            if (method.equalsIgnoreCase("merge")) {
                TidyJobFactory jobFactory = new TidyJobFactory(systemMetadataDao,
                        mergeResultsRepository, changeRecordRepository, mnReadClient);

                MergeExecutorService executorService = new MergeExecutorService(systemMetadataDao,
                        mergeResultsRepository, jobFactory);

                executorService.executeJobs();
            } else if (method.equalsIgnoreCase("evict")) {
                // connect to hazelcast and evict all systemMeta, if you dare
                PostMerge postMergeProcess = new PostMerge(mergeResultsRepository);
                postMergeProcess.postMerge();
            }
            repositoryConfig.closeContext();
        } catch (ParseException ex) {
            logger.error(ex.getMessage());
        }
    }
}
